#include <EEPROM.h>                                                // Библиотека для работы с EEPROM
#include "ASOLED.h"

//------------------------------------ Настройка подключения пинов к ардуино

#define detect 3                                                   // Детектор нуля
#define triac 5                                                    // Управление симистором
#define SW 7                                                       // энкодер SW кнопка
#define CLK 8                                                      // энкодер CLK
#define DT 9                                                       // энкодер DT
#define STP 10                                                     // кнопка стоп
#define RZG 11                                                     // кнопка разгон

//------------------------------------ Константы

double resist_ten_max = 650.00;                                    // Максимальное сопротивление ТЭНа, Ом
double resist_ten_min = 5.00;                                     // Минимальное сопротивление ТЭНа, Ом
double I_max_max = 30.00;                                         // Максимальный ограничивающй ток, А
double I_max_min = 5.00;                                          // Миниимальный ограничивающй ток, А

//------------------------------------ Переменные настроек

double resist_ten;                                                // Сопротивление ТЭНа
int power_ten;                                                    // Расчётная мощность ТЭНа
int power_ten_max = 220.00 * 220.00 / resist_ten_min;             // Максимальная расчетная мощность ТЭНа
int power_ten_min = 220.00 * 220.00 / resist_ten_max;             // Минимальная расчетная мощность ТЭНа
double I_max;                                                     // Ограничивающий ток
int P_max;                                                        // Ограничивающая мощность
int P_max_max;                                                    // Максимальная ограничивающая мощность
int P_max_min;                                                    // Минимальная ограничивающая мощность
int ACS_type;                                                     // Тип датчика ACS712
double ACS_coeff;                                                 // Коэффициент датчика ACS712 |5А - 0.024414063 | 20А - 0.048828125 | 30A - 0.073242188 |
int P_ust1;                                                       // Предустановка мощности 1
int P_ust2;                                                       // Предустановка мощности 2
int P_ust3;                                                       // Предустановка мощности 3
int P_ust4;                                                       // Предустановка мощности 4
byte setup_sw = false;                                            // Флаг режима настройки
int setup_step = 1;                                               // Флаг настройки текущего параметра
int menu_step = 0;                                                // Флаг меню предустановок
int ust_W = 0;                                                    // Установленная мощность
int angle = 2250;                                                 // Угол открытия симистора

//------------------------------------ Переменные для расчета RMS тока и мощности

volatile unsigned long Iism = 0;                                  // Мгновенное значение тока
volatile int cntr = 0;                                            // Счетчик в обработчике прерывания
volatile unsigned long Isumm = 0;                                 // Сумма квадратов тока
double real_I = 0;                                                // Расчетный RMS ток
int real_W = 0;                                                   // Расчетная мощность
double sqrt_I_sum = 0;                                            // Переменная для расчета квадратного корня
int zero = 0;                                                     // Детект нуля

//------------------------------------ Переменные для таймингов

unsigned long previousMillis = 0;                                 // Момент времени для регулирования мощности
unsigned long displayMillis = 0;                                  // Момент времени для обновления LCD
unsigned long encoderMillis = 0;                                  // Момент времени для энкодера

//------------------------------------ Переменные для энкодера

unsigned long encoderMillis_speed = 0;                            // Скорость вращения энкодера
int encoder_CLK = 0;                                              // Обработка вращения
int encoder_DT = 0;                                               // Обработка вращения
int encoder_CLK_prev = 0;                                         // Обработка вращения
byte set_CLK = false;                                             // Обработка нажатия
unsigned long SW_time_press = 0;                                  // Момент времени нажатия
unsigned long SW_press_time = 0;                                  // Счетчик времени нажатия

//------------------------------------ Переменные для кнопок

bool sw_stp = false;                                              // Свитч кнопки стоп
bool sw_rzg = false;                                              // Свитч кнопки разгон
bool sw_stp_old = false;                                          // Свитч кнопки стоп
bool sw_rzg_old = false;                                          // Свитч кнопки разгон
unsigned long time_button = 0;                                    // Момент времени нажатия кнопки

//------------------------------------ Процедура инициализации

void setup(void) {

  //------------------------------------ Инициализация вводов-выводов

  pinMode(detect, INPUT);                                         // Инициализация входа детекта нуля
  pinMode(triac, OUTPUT);                                         // Инициализация выхода управления симистором
  pinMode(CLK, INPUT_PULLUP);                                     // Подтягивание резистора входа энкодера CLK
  pinMode(DT, INPUT_PULLUP);                                      // Подтягивание резистора входа энкодера DT
  pinMode(SW, INPUT_PULLUP);                                      // Подтягивание резистора входа энкодера SW
  pinMode(STP, INPUT);                                            // Инициализация кнопки стоп
  pinMode(RZG, INPUT);                                            // Инициализация кнопки разгон
  
  //------------------------------------ Инициализация OLED

  LD.init();
  LD.clearDisplay();
  LD.printString_12x16(F("Регулятор"), 10, 2);
  LD.printString_12x16(F("мощности"), 16, 4);
  delay(1000);
  LD.clearDisplay();

  //------------------------------------ Считывание настроек из EEPROM

  EEPROM.get(0, resist_ten);
  if (isnan(resist_ten) || resist_ten > resist_ten_max || resist_ten < resist_ten_min)
  {
    resist_ten = resist_ten_min;
    setup_sw = true;
  }
  power_ten = 220.00 * 220.00 / resist_ten;

  EEPROM.get(4, I_max);
  if (isnan(I_max) || I_max > I_max_max || I_max < I_max_min)
  {
    I_max = I_max_min;
    setup_sw = true;
  }
  P_max = round(I_max * I_max * resist_ten / 5) * 5;
  P_max_max = round(I_max_max * I_max_max * resist_ten / 5) * 5;
  P_max_min = round(I_max_min * I_max_min * resist_ten / 5) * 5;

  EEPROM.get(8, ACS_type);
  if (isnan(ACS_type) || ACS_type > 2 || ACS_type < 0)
  {
    ACS_type = 0;
    setup_sw = true;
  }
  switch (ACS_type)
  {
    case 0:
      ACS_coeff = 0.024414063;
      break;
    case 1:
      ACS_coeff = 0.048828125;
      break;
    case 2:
      ACS_coeff = 0.073242188;
      break;
  }

  EEPROM.get(10, P_ust1);
  if (isnan(P_ust1) || P_ust1 < 0)
  {
    P_ust1 = 0;
    setup_sw = true;
  }
  EEPROM.get(12, P_ust2);
  if (isnan(P_ust2) || P_ust2 < 0)
  {
    P_ust2 = 0;
    setup_sw = true;
  }
  EEPROM.get(14, P_ust3);
  if (isnan(P_ust3) || P_ust3 < 0)
  {
    P_ust3 = 0;
    setup_sw = true;
  }
  EEPROM.get(16, P_ust4);
  if (isnan(P_ust4) || P_ust4 < 0)
  {
    P_ust4 = 0;
    setup_sw = true;
  }

  if (setup_sw)
  {
    LD.printString_12x16(F("Настройки"), 16, 2);
    delay(1000);
    LD.clearDisplay();
    dispSet1();
  }





  //------------------------------------ Настройка АЦП

  ADMUX = (0 << REFS1) | (1 << REFS0) | (0 << MUX2) | (0 << MUX1) | (1 << MUX0);
  ADCSRA = B11101111;
  TCCR1A = 0x00;
  TCCR1B = 0x00;
  TCCR1B = (0 << CS12) | (1 << CS11) | (1 << CS10);
  OCR1A = 0;
  TIMSK1 |= (1 << OCIE1A) | (1 << TOIE1);


  //------------------------------------ Вызов прерывания при детектировании нуля

  attachInterrupt(1, zero_crosss_int, RISING);

}

//------------------------------------  Прерывание по нулю

void zero_crosss_int()
{
  TCNT1 = 0;
  OCR1A = angle;
  zero++;
}

//------------------------------------ Обработка таймера по совпадению нуля

ISR (TIMER1_COMPA_vect)
{
  PORTD |=  (1 << PORTD5);
  TCNT1 = 65535 - 200;
}

ISR (TIMER1_OVF_vect)
{
  PORTD &=  ~(1 << PORTD5);
  TCNT1 = OCR1A + 1;
}

//------------------------------------ Обработка прерывания АЦП для расчета среднеквадратичного тока

ISR(ADC_vect)
{
  byte An_pin = ADCL;
  byte An = ADCH;
  Iism = ((An << 8) + An_pin) - 512;
  Iism *= Iism;
  Isumm += Iism;
  cntr++;
}

void loop(void) {

  //------------------------------------ Расчет RMS тока и мощности

  if (zero >= 8)                                               //zero - количество полупериодов для рассчета среднеквадратичного
  {
    sqrt_I_sum = Isumm / cntr; //
    real_I = sqrt(sqrt_I_sum) * ACS_coeff; //
    real_W = real_I * real_I * resist_ten;
    cntr = 0;
    Isumm = 0;
    zero = 0;
  }

  //------------------------------------ Обработка кнопки энкодера

  if (digitalRead(SW) == LOW) {
    if (millis() - SW_time_press > 1) {
      SW_press_time++;
      SW_time_press = millis();
    }
  }

  if (digitalRead(SW) == HIGH) {

    if ( SW_press_time >= 30 & !set_CLK)
    {
      if (setup_sw)
      {
        switch (setup_step) {
          case 1:
            setup_step = 2;
            set_CLK = true;
            dispSet1();
            break;
          case 2:
            setup_step = 3;
            set_CLK = true;
            P_max = round(I_max * I_max * resist_ten / 5) * 5;
            P_max_max = round(I_max_max * I_max_max * resist_ten / 5) * 5;
            P_max_min = round(I_max_min * I_max_min * resist_ten / 5) * 5;
            dispSet1();
            break;
          case 3:
            setup_step = 4;
            set_CLK = true;
            dispSet1();
            break;
          case 4:
            setup_step = 5;
            set_CLK = true;
            if (P_ust1 > P_max ) P_ust1 = P_max;
            if (P_ust2 > P_max ) P_ust2 = P_max;
            if (P_ust3 > P_max ) P_ust3 = P_max;
            if (P_ust4 > P_max ) P_ust4 = P_max;
            LD.clearDisplay();
            dispSet2();
            break;
          case 5:
            setup_step = 6;
            set_CLK = true;
            dispSet2();
            break;
          case 6:
            setup_step = 7;
            set_CLK = true;
            dispSet2();
            break;
          case 7:
            setup_step = 8;
            set_CLK = true;
            dispSet2();
            break;
          case 8:
            setup_step = 9;
            set_CLK = true;
            LD.clearDisplay();
            dispSet3();
            break;
          case 9:
            setup_step = 1;
            set_CLK = true;
            LD.clearDisplay();
            dispSet1();
            break;
        }
      }
      else
      {
        switch (menu_step)
        {
          case 0:
            menu_step = 1;
            set_CLK = true;
            dispMain();
            break;
          case 1:
            ust_W = P_ust1;
            menu_step = 0;
            set_CLK = true;
            dispMain();
            break;
          case 2:
            ust_W = P_ust2;
            menu_step = 0;
            set_CLK = true;
            dispMain();
            break;
          case 3:
            ust_W = P_ust3;
            menu_step = 0;
            set_CLK = true;
            dispMain();
            break;
          case 4:
            ust_W = P_ust4;
            menu_step = 0;
            set_CLK = true;
            dispMain();
            break;
        }
      }
    }

    SW_press_time = 0;
    set_CLK = false;
  }

  if (SW_press_time > 1000) {
    if (setup_sw)
    {
      EEPROM.put(0, resist_ten);
      EEPROM.put(4, I_max);
      EEPROM.put(8, ACS_type);
      EEPROM.put(10, P_ust1);
      EEPROM.put(12, P_ust2);
      EEPROM.put(14, P_ust3);
      EEPROM.put(16, P_ust4);
      setup_sw = false;

      LD.clearDisplay();
      LD.printString_12x16(F("Настройки"), 10, 2);
      LD.printString_12x16(F("сохранены"), 10, 4);
      delay(1000);
      LD.clearDisplay();

    }
    else
    {
      angle = 2250;
      ust_W = 0;
      setup_sw = true;
      setup_step = 1;

      LD.clearDisplay();
      LD.printString_12x16(F("Настройки"), 10, 2);
      delay(1000);
      LD.clearDisplay();
      dispSet1();

    }

    SW_press_time = 0;
    set_CLK = true;
  }


  //------------------------------------ Обработка вращения энкодера

  if (millis() >= (encoderMillis + 5))
  {
    encoder_CLK = digitalRead(CLK);
    encoder_DT = digitalRead(DT);
    if ((!encoder_CLK) && (encoder_CLK_prev))
    {
      if (encoder_DT)
      {
        if (setup_sw)
        {
          switch (setup_step)
          {
            case 1:                                   // Настройка сопротивления ТЭНа
              resist_ten += (millis() <= (encoderMillis_speed + 30)) ? 2.00 : (millis() <= (encoderMillis_speed + 70)) ? 1.00 : (millis() <= (encoderMillis_speed + 270)) ? 0.10 : 0.01;
              if (resist_ten > resist_ten_max ) resist_ten = resist_ten_max;
              power_ten = 220.00 * 220.00 / resist_ten;
              dispSet1();
              break;
            case 2:                                   // Настройка мощности ТЭНа
              power_ten += (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 10 : 1;
              if (power_ten > power_ten_max ) power_ten = power_ten_max;
              resist_ten = 220.00 * 220.00 / (double)power_ten;
              dispSet1();
              break;
            case 3:                                   // Настройка ограничивающего тока
              I_max += (millis() <= (encoderMillis_speed + 30)) ? 2.00 : (millis() <= (encoderMillis_speed + 70)) ? 1.00 : (millis() <= (encoderMillis_speed + 270)) ? 0.10 : 0.01;
              if (I_max > I_max_max ) I_max = I_max_max;
              P_max = round(I_max * I_max * resist_ten / 5) * 5;
              dispSet1();
              break;
            case 4:                                   // Настройка ограничивающей мощности
              P_max += (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_max > P_max_max ) P_max = P_max_max;
              I_max = sqrt((double)P_max / resist_ten);
              dispSet1();
              break;
            case 5:                                   // Настройка предустановок мощности
              P_ust1 += (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_ust1 > P_max ) P_ust1 = P_max;
              dispSet2();
              break;
            case 6:
              P_ust2 += (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_ust2 > P_max ) P_ust2 = P_max;
              dispSet2();
              break;
            case 7:
              P_ust3 += (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_ust3 > P_max ) P_ust3 = P_max;
              dispSet2();
              break;
            case 8:
              P_ust4 += (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_ust4 > P_max ) P_ust4 = P_max;
              dispSet2();
              break;
            case 9:                                   // Настройка типа датчика
              switch (ACS_type)
              {
                case 0:
                  ACS_type = 1;
                  ACS_coeff = 0.048828125;
                  break;
                case 1:
                  ACS_type = 2;
                  ACS_coeff = 0.073242188;
                  break;
                case 2:
                  ACS_type = 0;
                  ACS_coeff = 0.024414063;
                  break;
              }
              dispSet3();
              break;
          }
        }
        else
        { // Настройка подаваемой мощности
          switch (menu_step)
          {
            case 0:
              ust_W +=  (millis() <= (encoderMillis_speed + 50)) ? 50 :  (millis() <= (encoderMillis_speed + 200)) ? 10 : 5;
              if (ust_W > P_max) ust_W = P_max;
              dispMain();
              break;
            case 1:
              menu_step = 2;
              dispMain();
              break;
            case 2:
              menu_step = 3;
              dispMain();
              break;
            case 3:
              menu_step = 4;
              dispMain();
              break;
            case 4:
              menu_step = 0;
              dispMain();
              break;
          }
        }
        encoderMillis_speed = millis();
      }
      else {
        if (setup_sw)
        {
          switch (setup_step)
          {
            case 1:                                   // Настройка сопротивления ТЭНа
              resist_ten -= (millis() <= (encoderMillis_speed + 30)) ? 2.00 : (millis() <= (encoderMillis_speed + 70)) ? 1.00 : (millis() <= (encoderMillis_speed + 270)) ? 0.10 : 0.01;
              if (resist_ten < resist_ten_min ) resist_ten = resist_ten_min;
              power_ten = 220.00 * 220.00 / resist_ten;
              dispSet1();
              break;
            case 2:                                   // Настройка мощности ТЭНа
              power_ten -= (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 10 : 1;
              if (power_ten < power_ten_min ) power_ten = power_ten_min;
              resist_ten = 220.00 * 220.00 / (double)power_ten;
              dispSet1();
              break;
            case 3:                                   // Настройка ограничивающего тока
              I_max -= (millis() <= (encoderMillis_speed + 30)) ? 2.00 : (millis() <= (encoderMillis_speed + 70)) ? 1.00 : (millis() <= (encoderMillis_speed + 270)) ? 0.10 : 0.01;
              if (I_max < I_max_min ) I_max = I_max_min;
              P_max = round(I_max * I_max * resist_ten / 5) * 5;
              dispSet1();
              break;
            case 4:                                   // Настройка ограничивающей мощности
              P_max -= (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_max < P_max_min ) P_max = P_max_min;
              I_max = sqrt((double)P_max / resist_ten);
              dispSet1();
              break;
            case 5:                                   // Настройка предустановок мощности
              P_ust1 -= (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_ust1 < 0 ) P_ust1 = 0;
              dispSet2();
              break;
            case 6:
              P_ust2 -= (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_ust2 < 0 ) P_ust2 = 0;
              dispSet2();
              break;
            case 7:
              P_ust3 -= (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_ust3 < 0 ) P_ust3 = 0;
              dispSet2();
              break;
            case 8:
              P_ust4 -= (millis() <= (encoderMillis_speed + 30)) ? 200 : (millis() <= (encoderMillis_speed + 70)) ? 100 : (millis() <= (encoderMillis_speed + 270)) ? 20 : 5;
              if (P_ust4 < 0 ) P_ust4 = 0;
              dispSet2();
              break;
            case 9:                                   // Настройка типа датчика
              switch (ACS_type)
              {
                case 0:
                  ACS_type = 2;
                  ACS_coeff = 0.073242188;
                  break;
                case 1:
                  ACS_type = 0;
                  ACS_coeff = 0.024414063;
                  break;
                case 2:
                  ACS_type = 1;
                  ACS_coeff = 0.048828125;
                  break;
              }
              dispSet3();
              break;
          }
        }
        else
        { // Настройка подаваемой мощности

          switch (menu_step)
          {
            case 0:
              ust_W -=  (millis() <= (encoderMillis_speed + 50)) ? 50 :  (millis() <= (encoderMillis_speed + 300)) ? 10 : 5;
              if (ust_W < 0) ust_W = 0;
              dispMain();
              break;
            case 1:
              menu_step = 0;
              dispMain();
              break;
            case 2:
              menu_step = 1;
              dispMain();
              break;
            case 3:
              menu_step = 2;
              dispMain();
              break;
            case 4:
              menu_step = 3;
              dispMain();
              break;
          }

        }
        encoderMillis_speed = millis();
      }

    }
    encoder_CLK_prev = encoder_CLK;

    encoderMillis = millis();
  }

  //------------------------------------ Обработка нажатия кнопок

  if (digitalRead(STP) == HIGH) {
    if (!sw_stp_old) {
      time_button = millis();
      sw_stp_old = true;
    }
    if (millis() - time_button > 300) {
      sw_stp = true;
      sw_rzg = false;
      ust_W = 0;
    }
  } else {
    sw_stp_old = false;

    if (digitalRead(RZG) == HIGH) {
      if (!sw_rzg_old) {
        time_button = millis();
        sw_rzg_old = true;
      }
      if (millis() - time_button > 300) {
        sw_rzg = true;
        ust_W = P_ust4;
      }

    } else {
      sw_rzg_old = false;
    }
  }

  if (millis() - time_button > 2000) {
    sw_stp = false;
    sw_rzg = false;
  }

  //------------------------------------ Регулятор мощности

  if (millis() - previousMillis >= 15) {              //Если регулировка долгая то уменьшить значение, если видно частые изменения регулировки то увеличить значение
    previousMillis = millis();
    if (real_W > ust_W + 5) angle++;
    if (real_W < ust_W - 5) angle--;
    if (real_W < ust_W + 5 && real_W > ust_W - 5) angle = angle;
    if (angle > 2250) angle = 2250;
    if (angle <= 1) angle = 1;
    if (ust_W == 0) angle = 2250;
  }

  if (ust_W == 0) {
    TIMSK1 = 0x00;
    PORTD &=  ~(1 << PORTD5);
  }

  else
  {
    TIMSK1 |= (1 << OCIE1A) | (1 << TOIE1);
  }

  //------------------------------------ Вывод информации на LCD

  if (millis() - displayMillis >= 300) {
    displayMillis = millis();

    if (!setup_sw)
    {
      dispMain();
    }
  }
}

void dispSet1()
{
  LD.setFont(Font_6x8);
  LD.printString(F("Настройки тэна"), 20, 0);

  switch (setup_step)
  {
    case 1:
      LD.printString(F(">R тэн"), 0, 2);
      LD.printString(F(" W тэн"), 0, 3);
      LD.printString(F(" I макс"), 0, 4);
      LD.printString(F(" W макс"), 0, 5);
      break;
    case 2:
      LD.printString(F(" R тэн"), 0, 2);
      LD.printString(F(">W тэн"), 0, 3);
      LD.printString(F(" I макс"), 0, 4);
      LD.printString(F(" W макс"), 0, 5);
      break;
    case 3:
      LD.printString(F(" R тэн"), 0, 2);
      LD.printString(F(" W тэн"), 0, 3);
      LD.printString(F(">I макс"), 0, 4);
      LD.printString(F(" W макс"), 0, 5);
      break;
    case 4:
      LD.printString(F(" R тэн"), 0, 2);
      LD.printString(F(" W тэн"), 0, 3);
      LD.printString(F(" I макс"), 0, 4);
      LD.printString(F(">W макс"), 0, 5);
      break;

  }
  LD.printNumber(resist_ten, 2, 60, 2);
  if (resist_ten < 10) {
    LD.printString(F("   "));
  } else if (resist_ten < 100) {
    LD.printString(F("  "));
  } else {
    LD.printString(F(" "));
  }
  LD.printNumber(power_ten, 0, 60, 3);
  if (power_ten < 10) {
    LD.printString(F("    "));
  } else if (power_ten < 100) {
    LD.printString(F("   "));
  } else if (power_ten < 1000) {
    LD.printString(F("  "));
  }

  LD.printNumber(I_max, 2, 60, 4);
  LD.printString(F("   "));

  LD.printNumber(P_max, 0, 60, 5);
  if (P_max < 10) {
    LD.printString(F("    "));
  } else if (P_max < 100) {
    LD.printString(F("   "));
  } else if (P_max < 1000) {
    LD.printString(F("  "));
  } else if (P_max < 10000) {
    LD.printString(F(" "));
  }
  LD.printString(F("Ом"), 110, 2);
  LD.printString(F("Вт"), 110, 3);
  LD.printString(F("А"), 110, 4);
  LD.printString(F("Вт"), 110, 5);
}

void dispSet2()
{
  LD.setFont(Font_6x8);
  LD.printString(F("Настройка уставок"), 10, 0);

  switch (setup_step)
  {
    case 5:
      LD.printString(F(">P1"), 20, 2);
      LD.printString(F(" P2"), 20, 3);
      LD.printString(F(" P3"), 20, 4);
      LD.printString(F(" P4"), 20, 5);
      break;
    case 6:
      LD.printString(F(" P1"), 20, 2);
      LD.printString(F(">P2"), 20, 3);
      LD.printString(F(" P3"), 20, 4);
      LD.printString(F(" P4"), 20, 5);
      break;
    case 7:
      LD.printString(F(" P1"), 20, 2);
      LD.printString(F(" P2"), 20, 3);
      LD.printString(F(">P3"), 20, 4);
      LD.printString(F(" P4"), 20, 5);
      break;
    case 8:
      LD.printString(F(" P1"), 20, 2);
      LD.printString(F(" P2"), 20, 3);
      LD.printString(F(" P3"), 20, 4);
      LD.printString(F(">P4"), 20, 5);
      break;

  }

  LD.printNumber(P_ust1, 0, 50, 2);
  if (P_ust1 < 10) {
    LD.printString(F("    "));
  } else if (P_ust1 < 100) {
    LD.printString(F("   "));
  } else if (P_ust1 < 1000) {
    LD.printString(F("  "));
  }

  LD.printNumber(P_ust2, 0, 50, 3);
  if (P_ust2 < 10) {
    LD.printString(F("    "));
  } else if (P_ust2 < 100) {
    LD.printString(F("   "));
  } else if (P_ust2 < 1000) {
    LD.printString(F("  "));
  }

  LD.printNumber(P_ust3, 0, 50, 4);
  if (P_ust3 < 10) {
    LD.printString(F("    "));
  } else if (P_ust3 < 100) {
    LD.printString(F("   "));
  } else if (P_ust3 < 1000) {
    LD.printString(F("  "));
  }

  LD.printNumber(P_ust4, 0, 50, 5);
  if (P_ust4 < 10) {
    LD.printString(F("    "));
  } else if (P_ust4 < 100) {
    LD.printString(F("   "));
  } else if (P_ust4 < 1000) {
    LD.printString(F("  "));
  }

  LD.printString(F("Вт"), 90, 2);
  LD.printString(F("Вт"), 90, 3);
  LD.printString(F("Вт"), 90, 4);
  LD.printString(F("Вт"), 90, 5);

}

void dispSet3()
{
  LD.setFont(Font_6x8);
  LD.printString(F("Тип датчика тока"), 10, 0);
  switch (ACS_type)
  {
    case 0:
      LD.printString(F("ACS712 5A "), 30, 2);
      break;
    case 1:
      LD.printString(F("ACS712 20A"), 30, 2);
      break;
    case 2:
      LD.printString(F("ACS712 30A"), 30, 2);
      break;
  }
}

void dispMain()
{
  LD.setFont(Font_12x16);
  LD.printNumber(ust_W, 1, 5, 0);

  if (ust_W < 10) {
    LD.printString(F("   "));
  } else if (ust_W < 100) {
    LD.printString(F("  "));
  } else if (ust_W < 1000) {
    LD.printString(F(" "));
  }

  LD.setFont(Font_6x8);
  LD.printNumber(real_W, 0, 80, 0);
  if (real_W < 10) {
    LD.printString(F(" Вт   "));
  } else if (real_W < 100) {
    LD.printString(F(" Вт  "));
  } else if (real_W < 1000) {
    LD.printString(F(" Вт "));
  } else {
    LD.printString(F(" Вт"));
  }
  LD.printNumber(floor(real_I * 10 + 0.5) / 10, 1, 80, 1);
  LD.printString(F(" А  "));
  LD.printNumber(map(angle, 2250, 1, 0, 100), 0, 80, 2);
  LD.printString(F(" %  "));

  if (sw_stp) {
    LD.printString(F("Стоп"), 3, 2);
  } else if (sw_rzg) {
    LD.printString(F("Разгон"), 3, 2);
  } else {
    LD.printString(F("      "), 3, 2);
  }

  LD.setFont(Font_12x16);
  switch (menu_step)
  {
    case 1:
      LD.printString(F(">"), 0, 4);
      LD.printString(F(" "), 0, 6);
      LD.printString(F(" "), 63, 4);
      LD.printString(F(" "), 63, 6);
      LD.printNumber(P_ust1, 0, 15, 4);
      LD.printNumber(P_ust2, 0, 15, 6);
      LD.printNumber(P_ust3, 0, 78, 4);
      LD.printNumber(P_ust4, 0, 78, 6);
      break;
    case 2:
      LD.printString(F(" "), 0, 4);
      LD.printString(F(">"), 0, 6);
      LD.printString(F(" "), 63, 4);
      LD.printString(F(" "), 63, 6);
      LD.printNumber(P_ust1, 0, 15, 4);
      LD.printNumber(P_ust2, 0, 15, 6);
      LD.printNumber(P_ust3, 0, 78, 4);
      LD.printNumber(P_ust4, 0, 78, 6);
      break;
    case 3:
      LD.printString(F(" "), 0, 4);
      LD.printString(F(" "), 0, 6);
      LD.printString(F(">"), 63, 4);
      LD.printString(F(" "), 63, 6);
      LD.printNumber(P_ust1, 0, 15, 4);
      LD.printNumber(P_ust2, 0, 15, 6);
      LD.printNumber(P_ust3, 0, 78, 4);
      LD.printNumber(P_ust4, 0, 78, 6);
      break;
    case 4:
      LD.printString(F(" "), 0, 4);
      LD.printString(F(" "), 0, 6);
      LD.printString(F(" "), 63, 4);
      LD.printString(F(">"), 63, 6);
      LD.printNumber(P_ust1, 0, 15, 4);
      LD.printNumber(P_ust2, 0, 15, 6);
      LD.printNumber(P_ust3, 0, 78, 4);
      LD.printNumber(P_ust4, 0, 78, 6);
      break;
    default:
      LD.printString(F("           "), 0, 4);
      LD.printString(F("           "), 0, 6);
  }

}
